@@ -21,7 +21,7 @@ def create_parser():
     """Create argument parser for ONVIF CLI"""
     parser = argparse.ArgumentParser(
         prog="onvif",
-        description=f"{colorize('ONVIF Terminal Client', 'yellow')} — v0.2.0\nhttps://github.com/nirsimetri/onvif-python",
+        description=f"{colorize('ONVIF Terminal Client', 'yellow')} — v0.2.2\nhttps://github.com/nirsimetri/onvif-python",
         formatter_class=argparse.RawDescriptionHelpFormatter,
         epilog=f"""
 Examples:
@@ -198,10 +198,14 @@ def search_products(search_term: str, page: int = 1, per_page: int = 20) -> None
         WHERE LOWER(model) LIKE LOWER(?)
            OR LOWER(post_title) LIKE LOWER(?)
            OR LOWER(company_name) LIKE LOWER(?)
+           OR LOWER(product_category) LIKE LOWER(?)
         """
 
         search_pattern = f"%{search_term}%"
-        cursor.execute(count_query, (search_pattern, search_pattern, search_pattern))
+        cursor.execute(
+            count_query,
+            (search_pattern, search_pattern, search_pattern, search_pattern),
+        )
         total_count = cursor.fetchone()[0]
 
         if total_count == 0:
@@ -229,12 +233,21 @@ def search_products(search_term: str, page: int = 1, per_page: int = 20) -> None
         WHERE LOWER(model) LIKE LOWER(?)
            OR LOWER(post_title) LIKE LOWER(?)
            OR LOWER(company_name) LIKE LOWER(?)
+           OR LOWER(product_category) LIKE LOWER(?)
         ORDER BY test_date DESC
         LIMIT ? OFFSET ?
         """
 
         cursor.execute(
-            query, (search_pattern, search_pattern, search_pattern, per_page, offset)
+            query,
+            (
+                search_pattern,
+                search_pattern,
+                search_pattern,
+                search_pattern,
+                per_page,
+                offset,
+            ),
         )
         results = cursor.fetchall()
 
@@ -448,7 +461,7 @@ def main():
 
     # Show ONVIF CLI version
     if args.version:
-        print(colorize("0.2.0", "yellow"))
+        print(colorize("0.2.2", "yellow"))
         sys.exit(0)
 
     # Handle product search
